@@ -2,7 +2,7 @@ package PDF::Tiny;
 
 use 5.01;
 
-$VERSION = '0.02'; # Update the POD, too!
+$VERSION = '0.03'; # Update the POD, too!
 
 # Fields
 sub vers () { 0 }
@@ -66,9 +66,9 @@ sub new {
   }
   $self->[vers] = $1;
   _parse_xref($self);
+  $self->[size] = $self->[trai][1]{Size}[1];
   if ($self->[trai][1]{ID}) {
    $self->[id] = $self->[trai][1]{ID}[1][0][1];
-   $self->[size] = $self->[trai][1]{Size}[1];
   }
  }
  else {
@@ -713,32 +713,16 @@ sub tokenize { # This function tokenizes.
 	 		push @tokens, $1;
 		}
 		if (s.^\(..) {  # remove paren.
-			s/^((?:\\.|[^\)\\])*)//s;# get up to closing paren.
-			push @tokens, "($1";
-		
-				# count opening parens.:
-			while(my $count = () = $1 =~ 
-				/\G((?:
-				    \\.     # backslash-something
-					|   # or
-				    [^\(\\] # anything except \ and (
-				  )*
-				  \(   	    # opening parenthesis
-				)/sgx)
-			{
-				# match the same number of close parens.:
-				s/^((?:(?:\\.|[^\\\)])*\)){$count}
-					# non-')'* followed by ')'
-				    (?:\\.|[^\\\)])*
-					# non-')'*
-				)//sx;
-
-				# add to existing token:
-				$tokens[$#tokens] .= $1;
-			}
-			$tokens[$#tokens] .= ')';
-			$_ = substr $_, 1;
+			$more and &$more()
+				until s/(
+				 (?:\\.|[^()\\])++# escaped char or non-\()
+				  |
+				 \((?1)\) # parenthesized stuff
+				)*\) # final closing paren
+				//xs;
+			push @tokens, "($1)";
 		}
+		&$more() while /^<[^>]*\z/;
 	}
 	return @tokens;
     }
@@ -753,7 +737,8 @@ sub join_tokens {
  for (@_) {
   # We assume that only strings can get too long to fit on a line.  After
   # all, they are the only token that can be split between lines.
-  if (/^\$S*([(<])/o && length() + length $line > 254) {
+  if (/^$S*([(<])/o && length() + length $line > 254) {
+   my $hex = $1 eq '<';
    # Put a line break before the string.
    $ret .= "$line\n";
    $line = '';
@@ -764,7 +749,7 @@ sub join_tokens {
      # Don’t split it between an escaper and an escapee.
      my $piecepiece = substr $_, 0, 253;
      chop $piecepiece unless $piecepiece =~ /^[^\\]*(?:\\.[^\\]*)*\z/s;
-     $ret .= "$piecepiece\\\n";
+     $ret .= $hex ? "$piecepiece\n" : "$piecepiece\\\n";
      substr $_, 0, length $piecepiece, = '';
    }
    $ret .= "$_\n";
@@ -774,12 +759,14 @@ sub join_tokens {
    # complicated string-splitting logic above. (Speeeeeeeed!) (I hope.)
    # PDF::Extract won’t be able to read it.  That’s the least of
    # its problems.
-   if (length($line) and $line !~ /$D\z/o && !/^$D/o
-                       ||length($line) + length > 254) {
-    $ret .= "$line\n";
-    $line = '';
+   for (ref eq 'SCALAR' ? $$_ : $_) {
+    if (length($line) and $line !~ /$D\z/o && !/^$D/o
+                        ||length($line) + length > 254) {
+     $ret .= "$line\n";
+     $line = '';
+    }
+    $line .= $_;
    }
-   $line .= $_;
   }
  }
  "$ret$line";
@@ -840,9 +827,9 @@ sub _interpret_token { # pass an array ref
 				#i.e., if it's a string
 		s/\)$//; # remove final ")"
 		# and remove wack escapes:
-		s,$N|\\($N|\d{1\,3}|.),  my $match = $1;
-					my $fullmatch = $&;
-			$fullmatch =~ /$N/o ? "\cj" :	# EOL
+		s,($N)|\\($N|\d{1\,3}|.),  my $match = $2;
+					my $unescaped = $1;
+			$unescaped    ? "\cj"  :	# EOL
 			$match =~ /$N/o ? '' :		# \EOL
 			$match=~/\d/?chr oct$match :	# octal
 			$match eq 'n' ? "\cj" :		# CR
@@ -945,7 +932,7 @@ sub _serialize {
 	if($$_[0]eq 'str'){
 		 # copy it so we don't modify the object being flattened
 		for (my $ret = $$_[1]) {
-			s/([\\()])/\\$1/g;
+			s/([\\()\r])/$1 eq "\r" ? '\r' : "\\$1"/ge;
 			return"($_)";
 		}
 	}
@@ -988,7 +975,7 @@ sub _serialize {
 	
 	$$_[0]eq'null'&&return'null';
 	
-	$$_[0]eq'flat'&&return$$_[1];
+	$$_[0]eq'flat'&&return\$$_[1];
 	
 	$$_[0]eq'tokens'&&return@{$$_[1]};
 	
